<?php  
 /**
 * CakePHP Patch: Extending CakePHP's CacheHelper to use Cache Engines
 * http://www.datumbox.com/
 *
 * Copyright 2013, Vasilis Vryniotis
 * Licensed under MIT or GPLv3, see LICENSE
 */
 
//In /app/View/Helper/MyCacheHelper.php we change the way that the cache is written
//=================================================================
  
App::uses('CacheHelper', 'View/Helper');
class MyCacheHelper extends CacheHelper {
     
    /**
    *
    * The below _writeFile() fuction is almost identical to the original with only difference that we don't write directly the files
    * but instead we use Cache::write() to do so.
    * 
    */
    protected function _writeFile($content, $timestamp, $useCallbacks = false) {
        //Below this point the code is the SAME as in CakePHP's method
        //============================================================
        $now = time();
  
        if (is_numeric($timestamp)) {
            $cacheTime = $now + $timestamp;
        } else {
            $cacheTime = strtotime($timestamp, $now);
        }
        $path = $this->request->here();
        if ($path === '/') {
            $path = 'home';
        }
        $prefix = Configure::read('Cache.viewPrefix');
        if ($prefix) {
            $path = $prefix . '_' . $path;
        }
        $cache = strtolower(Inflector::slug($path));
  
        if (empty($cache)) {
            return;
        }
        $cache = $cache . '.php';
        $file = '<!--cachetime:' . $cacheTime . '--><?php';
  
        if (empty($this->_View->plugin)) {
            $file .= "
            App::uses('{$this->_View->name}Controller', 'Controller');
            ";
        } else {
            $file .= "
            App::uses('{$this->_View->plugin}AppController', '{$this->_View->plugin}.Controller');
            App::uses('{$this->_View->name}Controller', '{$this->_View->plugin}.Controller');
            ";
        }
  
        $file .= '
                $request = unserialize(base64_decode(\'' . base64_encode(serialize($this->request)) . '\'));
                $response = new CakeResponse();
                $controller = new ' . $this->_View->name . 'Controller($request, $response);
                $controller->plugin = $this->plugin = \'' . $this->_View->plugin . '\';
                $controller->helpers = $this->helpers = unserialize(base64_decode(\'' . base64_encode(serialize($this->_View->helpers)) . '\'));
                $controller->layout = $this->layout = \'' . $this->_View->layout . '\';
                $controller->theme = $this->theme = \'' . $this->_View->theme . '\';
                $controller->viewVars = unserialize(base64_decode(\'' . base64_encode(serialize($this->_View->viewVars)) . '\'));
                Router::setRequestInfo($controller->request);
                $this->request = $request;';
  
        if ($useCallbacks) {
            $file .= '
                $controller->constructClasses();
                $controller->startupProcess();';
        }
  
        $file .= '
                $this->viewVars = $controller->viewVars;
                $this->loadHelpers();
                extract($this->viewVars, EXTR_SKIP);
        ?>';
        $content = preg_replace("/(<\\?xml)/", "<?php echo '$1'; ?>", $content);
        $file .= $content;
        //Up to this point the code is the SAME as in CakePHP's method
        //============================================================
            
        $cacheEngine='_cake_views_';
        $cacheKey = 'helper_cached_views_'.md5(CACHE . 'views' . DS . $cache);        
        if(!Cache::write($cacheKey, $file, $cacheEngine)) {
            return NULL;
        }
         
        return $file;
    }
}